use strict;
use warnings;

use Gtk2 -init;
use Glib qw /TRUE FALSE/;

# This is the size of the image in pixels in the png form
my @imagesize = (214, 148); 

# Coordinates of the hotspots. For simplicity sake they are rectangles here.
# But they could be any shape. You have to write the corresponding check_hotspot sub
my $hotspots = [
	[ [29, 24, 76, 54 ]  , ' Red Hot '  ],
	[ [90, 92, 170, 125] , ' Cool Green ' ],
];

# This is our little tooltip window.
my $tooltip_label = Gtk2::Label->new;
my $tooltip = Gtk2::Window->new('popup');
$tooltip->set_decorated(0);
$tooltip->set_position('mouse'); # We'll choose this to start with.
$tooltip->modify_bg ('normal', Gtk2::Gdk::Color->parse('yellow')); # The obligatory yellow
$tooltip->add($tooltip_label);
my $tooltip_displayed = FALSE;

# Read the image from a file.
my $image = Gtk2::Image->new_from_file('myimage.png');

# Image can't detect events. So load it into an eventbox explicitly.
my $eventbox = Gtk2::EventBox->new;
$eventbox->add ($image);
$eventbox->add_events (['pointer-motion-mask', 'pointer-motion-hint-mask']);
$eventbox->signal_connect ('motion-notify-event' => 
	sub {
		my ($widget, $event) = @_;
		my ($x, $y) = ($event->x, $event->y);
		# Since the image is smaller than the window, we need to translate these window coords
		# into the image coords.
		# Get the allocated size of the image first.
		# I assume that the image is always centered within the allocation.
		# I don't know if that assumption is buggy... ????
		# Then the coords are transformed.
		my @imageallocatedsize = $image->allocation->values;
		$x -= ($imageallocatedsize[2] - $imagesize[0])/2;
		$y -= ($imageallocatedsize[3] - $imagesize[1])/2;
		check_hotspot($x,$y);
	}
);

my $window = Gtk2::Window->new;
$window->signal_connect(destroy => sub { Gtk2->main_quit; });
$window->set_default_size(300, 200);
$window->add($eventbox);
$window->show_all;
Gtk2->main;

# Given a coord pair, this one will find whether the coord pair is in a hotspot or not
# This one assumes that the hotspots are rectangles.
sub check_hotspot {
	my ($x, $y) = @_;
	foreach my $hotspot (@$hotspots) {
		my ($coords, $desc) = @$hotspot;
		if ($x >= $coords->[0] && $x <= $coords->[2] && $y >= $coords->[1] && $y <= $coords->[3]) {
			$tooltip_label->set_label($desc);
			if (!$tooltip_displayed) {
				$tooltip->show_all;
				my ($thisx, $thisy) = $tooltip->window->get_origin;
				# I want the window to be a bit away from the mouse pointer.
				# Just a personal choice :)
				$tooltip->move($thisx, $thisy-20);
				$tooltip_displayed = TRUE;
			}
			return;
		} 
	}
	$tooltip->hide;
	$tooltip_displayed = FALSE;
}